==> RSTouchscreenNG-1.3.0.zip: RSTouchscreenNG/projects/front-ui/src/lib/modals/ui-visual-aids.service.ts <==
/**
 * Created by littwin on 10.06.16.
 */

import {Injectable, Component, ComponentRef} from "@angular/core";
import {VisualAidsBase, PrinterErrorAction, ProgressData} from "binding";
import type {RepetierState, PrintersState} from "binding";
import {RSModal} from "./rsModal.component";
import {RSModalViewComponent} from "./modal-view.component";
import {Observable} from "rxjs";
import {Store} from "@ngrx/store";
import type {PrinterErrorState} from "binding";
import {take} from "rxjs/operators";

@Component({
  selector: "rs-visual-aid-modal",
  template: `<div class="busy-modal"><h1 [innerHTML]="options.message"></h1></div>`
})
export class VisualAidModalComponent {
  options: any = {};
  hasError = false;

  constructor(public modal: RSModalViewComponent) {
    this.options = this.modal.data;
  }
}

@Component({
  selector: "rs-visual-aid-errors-modal",
  template: `<rs-modal-head>{{tPrinterErrors}}</rs-modal-head>
    <rs-modal-content rsDrag>
        <div *ngFor="let s of options.printerErrors">
            <h3>{{(printers$|async).printers[s.slug]?.name}}:</h3>
            <p>{{s.device}}: {{s.stateString}}</p>
        </div>
    </rs-modal-content>
    <rs-modal-footer><rs-button (hit)="ok()" class="error select full"><?php _('OK') ?></rs-button></rs-modal-footer>`
})
export class VisualAidErrorsModalComponent {
  options: any = {};
  tPrinterErrors = "<?php _('Printer Errors') ?>";
  printers$: Observable<PrintersState>;

  constructor(public modal: RSModalViewComponent,
              private store: Store<RepetierState>,
              private printerErrorAction: PrinterErrorAction) {
    this.printers$ = store.select('printers');
    this.options = this.modal.data;
  }

  ok() {
    this.printerErrorAction.markAllSeen();
  }
}

@Injectable()
export class RSUIVisualAids extends VisualAidsBase {

  progress: { [key: string]: string } = {};
  disconnectedOn = false;
  busyMsg: string[] = [];
  busyMsgType: string[] = []; // 0 = con, 1 = busy, progress id
  comp: ComponentRef<RSModalViewComponent> = undefined;
  compErrors: ComponentRef<RSModalViewComponent> = undefined;
  compCreated = (c: ComponentRef<RSModalViewComponent>) => {
    this.comp = c;
  }
  compErrorsCreated = (c: ComponentRef<RSModalViewComponent>) => {
    this.compErrors = c;
  }
  // tslint:disable-next-line
  errorOptions: any = {
    modalClass: "modal-error",
    componentCreatedCallback: this.compErrorsCreated,
    closeable: false,
    printerErrors: []
  };

  // tslint:disable-next-line
  options: any = {
    modalClass: "modal-noborder",
    closeable: false,
    componentCreatedCallback: this.compCreated
  };


  constructor(private modal: RSModal,
              private redux: Store<RepetierState>) {
    super();
  }

  get printerErrors(): { [key: string]: PrinterErrorState } {
    let pe: { [key: string]: PrinterErrorState };
    this.redux.select("printerErrors").pipe(take(1)).subscribe(printerErrors => {
      pe = printerErrors;
    });
    return pe;
  }

  onConnected(): void {
    if (this.disconnectedOn) {
      this.disconnectedOn = false;
      this._pop("0");
    }
  }

  onDisconnected(autoupdater: boolean): void {
    if (!this.disconnectedOn) {
      // console.log('connection lost');
      this.disconnectedOn = true;
      this.hideAll();
      if (autoupdater) {
        this._push("<i class=\"fa fa-refresh fa-spin fa-fw\"></i> <?php _('Autoupdate is currently updating the binaries. Please be patient.') ?>", "0");
      } else {
        this._push("<i class=\"fa fa-spinner fa-spin fa-fw\"></i> <?php _('Oops! I lost my connection to the server. Trying to reconnect.') ?>", "0");
      }
    }
  }

  private _push(reason: string, tp: string) {
    let idxFound = -1;
    for (let i = 0; i < this.busyMsgType.length; i++) {
      if (this.busyMsgType[i] === tp) {
        idxFound = i;
      }
    }
    if (idxFound < 0) {
      this.busyMsg.push(reason);
      this.busyMsgType.push(tp);
    } else {
      this.busyMsg[idxFound] = reason;
    }
    if (idxFound < 0 || idxFound === this.busyMsg.length - 1) {
      this.options.message = reason;
      if (this.busyMsg.length === 1 && idxFound < 0) {
        this.modal.show(VisualAidModalComponent, this.options).then(() => {
        }, () => {
        });
      }
    }
  }

  private _pop(tp: string) {
    if (this.busyMsg.length === 0) {
      return;
    }
    let idxFound = -1;
    for (let i = 0; i < this.busyMsgType.length; i++) {
      if (this.busyMsgType[i] === tp) {
        idxFound = i;
      }
    }
    if (idxFound < 0) {
      return;
    }

    this.busyMsgType.splice(idxFound, 1);
    this.busyMsg.splice(idxFound, 1);
    delete this.progress[tp];
    if (this.busyMsg.length === 0) {
      if (this.comp) {
        (<RSModalViewComponent>this.comp.instance).cancel();
      }
      this.comp = undefined;
    } else {
      this.options.message = this.busyMsg[this.busyMsg.length - 1];
    }
  }

  hideAll() {
    while (this.busyMsgType.length > 0) {
      this._pop(this.busyMsgType[0]);
    }
  }

  showBusy(reason: string): string {
    const id = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
      // tslint:disable-next-line
      const r = Math.random() * 16 | 0, v = c === "x" ? r : (r & 0x3 | 0x8);
      return v.toString(16);
    });
    this._push(reason, id);
    return id;
  }

  hideBusy(id: string): void {
    this._pop(id);
  }


  onPrinterError(): void {
    this.errorOptions.printerErrors = PrinterErrorAction.listErrors(this.printerErrors);
    if (!this.compErrors) {
      this.modal.show(VisualAidErrorsModalComponent, this.errorOptions).then(() => {
      }, () => {
      });
    }
  }

  onPrinterErrorSolved(): void {
    if (this.compErrors) {
      (<RSModalViewComponent>this.compErrors.instance).cancel();
    }
    this.compErrors = undefined;
  }

  updateProgress(p: ProgressData): void {
    // console.log("Progress", p);
    if (p.finished) {
      this._pop(p.msgId);
    } else {
      this._push(this.progress[p.msgId], p.msgId);
    }
  }

  bindProgressMsgToId(id: string, msg: string): void {
    this.progress[id] = msg;
  }

}

==> RSTouchscreenNG-1.4.16.zip: RSTouchscreenNG/projects/front-ui/src/lib/modals/ui-visual-aids.service.ts <==
/**
 * Created by littwin on 10.06.16.
 */

import {Injectable, Component, ComponentRef} from "@angular/core";
import {VisualAidsBase, PrinterErrorAction, ProgressData} from "binding";
import type {RepetierState, PrintersState} from "binding";
import {RSModal} from "./rsModal.component";
import {RSModalViewComponent} from "./modal-view.component";
import {Observable} from "rxjs";
import {Store} from "@ngrx/store";
import type {PrinterErrorState} from "binding";
import {take} from "rxjs/operators";

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-visual-aid-modal",
  template: `<div class="busy-modal"><h1 [innerHTML]="options.message"></h1></div>`
})
export class VisualAidModalComponent {
  options: any = {};
  hasError = false;

  constructor(public modal: RSModalViewComponent) {
    this.options = this.modal.data;
  }
}

@Component({
  // eslint-disable-next-line @angular-eslint/component-selector
  selector: "rs-visual-aid-errors-modal",
  template: `<rs-modal-head>{{tPrinterErrors}}</rs-modal-head>
    <rs-modal-content rsDrag>
        <div *ngFor="let s of options.printerErrors">
            <h3>{{(printers$|async).printers[s.slug]?.name}}:</h3>
            <p>{{s.device}}: {{s.stateString}}</p>
        </div>
    </rs-modal-content>
    <rs-modal-footer><rs-button (hit)="ok()" class="error select full"><?php _('OK') ?></rs-button></rs-modal-footer>`
})
export class VisualAidErrorsModalComponent {
  options: any = {};
  tPrinterErrors = "<?php _('Printer Errors') ?>";
  printers$: Observable<PrintersState>;

  constructor(public modal: RSModalViewComponent,
              private store: Store<RepetierState>,
              private printerErrorAction: PrinterErrorAction) {
    this.printers$ = store.select('printers');
    this.options = this.modal.data;
  }

  ok() {
    this.printerErrorAction.markAllSeen();
  }
}

@Injectable()
export class RSUIVisualAids extends VisualAidsBase {

  progress: { [key: string]: string } = {};
  disconnectedOn = false;
  busyMsg: string[] = [];
  busyMsgType: string[] = []; // 0 = con, 1 = busy, progress id
  comp: ComponentRef<RSModalViewComponent> = undefined;
  compErrors: ComponentRef<RSModalViewComponent> = undefined;
  compCreated = (c: ComponentRef<RSModalViewComponent>) => {
    this.comp = c;
  }
  compErrorsCreated = (c: ComponentRef<RSModalViewComponent>) => {
    this.compErrors = c;
  }
  // eslint-disable-next-line
  errorOptions: any = {
    modalClass: "modal-error",
    componentCreatedCallback: this.compErrorsCreated,
    closeable: false,
    printerErrors: []
  };

  // eslint-disable-next-line
  options: any = {
    modalClass: "modal-noborder",
    closeable: false,
    componentCreatedCallback: this.compCreated
  };


  constructor(private modal: RSModal,
              private redux: Store<RepetierState>) {
    super();
  }

  get printerErrors(): { [key: string]: PrinterErrorState } {
    let pe: { [key: string]: PrinterErrorState };
    this.redux.select("printerErrors").pipe(take(1)).subscribe(printerErrors => {
      pe = printerErrors;
    });
    return pe;
  }

  onConnected(): void {
    if (this.disconnectedOn) {
      this.disconnectedOn = false;
      this._pop("0");
    }
  }

  onDisconnected(autoupdater: boolean): void {
    if (!this.disconnectedOn) {
      // console.log('connection lost');
      this.disconnectedOn = true;
      this.hideAll();
      if (autoupdater) {
        this._push("<i class=\"fa fa-refresh fa-spin fa-fw\"></i> <?php _('Autoupdate is currently updating the binaries. Please be patient.') ?>", "0");
      } else {
        this._push("<i class=\"fa fa-spinner fa-spin fa-fw\"></i> <?php _('Oops! I lost my connection to the server. Trying to reconnect.') ?>", "0");
      }
    }
  }

  private _push(reason: string, tp: string) {
    let idxFound = -1;
    for (let i = 0; i < this.busyMsgType.length; i++) {
      if (this.busyMsgType[i] === tp) {
        idxFound = i;
      }
    }
    if (idxFound < 0) {
      this.busyMsg.push(reason);
      this.busyMsgType.push(tp);
    } else {
      this.busyMsg[idxFound] = reason;
    }
    if (idxFound < 0 || idxFound === this.busyMsg.length - 1) {
      this.options.message = reason;
      if (this.busyMsg.length === 1 && idxFound < 0) {
        this.modal.show(VisualAidModalComponent, this.options).then(() => {
        }, () => {
        });
      }
    }
  }

  private _pop(tp: string) {
    if (this.busyMsg.length === 0) {
      return;
    }
    let idxFound = -1;
    for (let i = 0; i < this.busyMsgType.length; i++) {
      if (this.busyMsgType[i] === tp) {
        idxFound = i;
      }
    }
    if (idxFound < 0) {
      return;
    }

    this.busyMsgType.splice(idxFound, 1);
    this.busyMsg.splice(idxFound, 1);
    delete this.progress[tp];
    if (this.busyMsg.length === 0) {
      if (this.comp) {
        (<RSModalViewComponent>this.comp.instance).cancel();
      }
      this.comp = undefined;
    } else {
      this.options.message = this.busyMsg[this.busyMsg.length - 1];
    }
  }

  hideAll() {
    while (this.busyMsgType.length > 0) {
      this._pop(this.busyMsgType[0]);
    }
  }

  showBusy(reason: string): string {
    const id = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
      // eslint-disable-next-line
      const r = Math.random() * 16 | 0, v = c === "x" ? r : (r & 0x3 | 0x8);
      return v.toString(16);
    });
    this._push(reason, id);
    return id;
  }

  hideBusy(id: string): void {
    this._pop(id);
  }


  onPrinterError(): void {
    this.errorOptions.printerErrors = PrinterErrorAction.listErrors(this.printerErrors);
    if (!this.compErrors) {
      this.modal.show(VisualAidErrorsModalComponent, this.errorOptions).then(() => {
      }, () => {
      });
    }
  }

  onPrinterErrorSolved(): void {
    if (this.compErrors) {
      (<RSModalViewComponent>this.compErrors.instance).cancel();
    }
    this.compErrors = undefined;
  }

  updateProgress(p: ProgressData): void {
    // console.log("Progress", p);
    if (p.finished) {
      this._pop(p.msgId);
    } else {
      this._push(this.progress[p.msgId], p.msgId);
    }
  }

  bindProgressMsgToId(id: string, msg: string): void {
    this.progress[id] = msg;
  }

}
